-----------------------------------------------------------------------------
--  Below Average Items - BelowAverageItems.lua
--
--  functions:
--		local function round(val, decimal)
--		local function bai_FindChatWindow(windowname)
--		local function bai_DebugOut(outString)
--		local function bai_UpdateEquippedLvl()
--		local function bai_UpdateEquippedItems()
--		local function bai_UpdateLowestItems()
--		function BAI_Indicator_OnEnter(self)
--		function BAI_OnShow(self)
--		function BAI_OnEvent(self, event, ...)
--		function BAI_OnLoad(self)
--
--  (c) 2012-2022 gmz323(Greg)
-----------------------------------------------------------------------------


-- MOP Changes
-- Removal of RangedSlot

-----------------------------------------------------------------------------
-- Constant and Local declarations
-----------------------------------------------------------------------------
local BAI_VERSION = "1.37"
local baiStartTime = GetTime()
local bai_currentAvgLvl = 0
local bai_currentLowItemCount = 0
local bai_slotNameArray = {"BackSlot","ChestSlot","FeetSlot","Finger0Slot","Finger1Slot","HandsSlot","HeadSlot","LegsSlot","MainHandSlot","NeckSlot","SecondaryHandSlot","ShoulderSlot","Trinket0Slot","Trinket1Slot","WaistSlot","WristSlot"}
-- temp table to hold slot item level and if image should be shown
local bai_slotItems = {} 
local bai_RefreshAvgLvl = 0
--Local pointer
local HiddenTooltip	= BAI_HiddenTooltip 

-- Debugging
local debugMode = false

local L = BAILocalization

-----------------------------------------------------------------------------
-- Rounds a number to the nearest decimal place.
--
-- @param val			value to be rounded.
-- @param decimal		decimal place to round to.
-- @return 				rounded value.
-----------------------------------------------------------------------------
local function round(val, decimal)
	if (decimal) then
		return math.floor( ((val * 10^decimal) + 0.5) / (10^decimal) )
	else
		return math.floor( val + 0.5 )
	end
end


-----------------------------------------------------------------------------
-- Slash command settings for client.
-----------------------------------------------------------------------------
SLASH_BAI1 = "/bai"
SLASH_BAI2 = "/belowaverageitems"

SlashCmdList["BAI"] = function(msg, editbox)
	print('|cFFFFFF00Below Average Items|r - addon version: v' .. BAI_VERSION)
end


-----------------------------------------------------------------------------
-- Looks for a window with passed in name. Returns 1 if not found.
--
-- @param windowname	name (on chat tab) of window to find.
-- @return 				window number to output debug messages to.
-----------------------------------------------------------------------------
local function bai_FindChatWindow(windowname)
	local x
	
	for x=1,NUM_CHAT_WINDOWS do
		local name, fontSize, r, g, b, alpha, shown, locked, docked, uninteractable = GetChatWindowInfo(x)
		if (name == windowname) then
			return x
		end	
	end
	-- Default to main chat window if DEBUG window not found
	return 1
end


-----------------------------------------------------------------------------
-- Handles formatting and output of debug strings.
--
-- @param outString		string to be output to debug window.
-----------------------------------------------------------------------------
local function bai_DebugOut(outString)
	if (debugMode) then
		-- calc delta time
		local prefixStr, deltaTime

		deltaTime = GetTime() - baiStartTime
		prefixStr = string.format("[%.3f] ", deltaTime)

		-- print to chatframe named "DEBUG" else main chatframe
		getglobal("ChatFrame"..(bai_FindChatWindow("DEBUG"))):AddMessage(prefixStr..outString)
	end
end


-----------------------------------------------------------------------------
-- Updates global variable bai_currentAvgLvl with AverageItemLevel when 
-- called.
-----------------------------------------------------------------------------
local function bai_UpdateEquippedLvl()

	-- IMPORTANT NOTE (2016/4/15): There appears to be certain pieces of gear
	-- that do NOT consistently trigger a PLAYER_EQUIPMENT_CHANGED event.
	-- This will cause an incorrect calculation of AverageItemLevel and the
	-- correct bai icons will not be displayed.

	local total_iLvl, equipped_iLevel, pvp_iLevel = GetAverageItemLevel()
	bai_DebugOut("GetAverageItemLevel() ="..total_iLvl..","..equipped_iLevel..","..pvp_iLevel)
	bai_currentAvgLvl = round(equipped_iLevel,2)
	
end


-----------------------------------------------------------------------------
-- Updates global bai_slotItems array with item level and if/which indicator
-- is shown.
-----------------------------------------------------------------------------
local function bai_UpdateEquippedItems()
	local i, slotName
	
	-- Loop through each item of array
	for i,slotName in ipairs(bai_slotNameArray) do 

		-- update item table
		bai_slotItems[slotName] = {}
		bai_slotItems[slotName]["indicator"] = 0 -- 0 hide / 1 show
		bai_slotItems[slotName]["iLvl"] = 0 -- slot iLvl, used in tooltip
		getglobal("BAI_"..slotName):Hide()
		
		local slotId, texture, checkRelic = GetInventorySlotInfo(slotName)
		local itemId = GetInventoryItemID("player", slotId)
		
		if (itemId ~= nil) then
			local name, _, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemId)

		
			local link = GetInventoryItemLink("player", slotId) -- get the item link for tooltip scan
			if (link ~= nil) then
				-- Use HiddenTooltip class to get real item level.
				local HTTiLevel = HiddenTooltip:GetEquipmentItemInfo(link, "ITEM_LEVEL")
				if (HTTiLevel ~= nil) then
					bai_DebugOut("BAI (slot/itemlevel): "..slotId.."/"..HTTiLevel)
					iLevel = tonumber(HTTiLevel)
				end
			end

			if (iLevel ~= nil) then

				bai_slotItems[slotName]["iLvl"] = iLevel

				if (iLevel < bai_currentAvgLvl) then
					bai_slotItems[slotName]["indicator"] = 1
					getglobal("BAI_"..slotName):Show()
					getglobal("BAI_"..slotName).yellow:Show()
					getglobal("BAI_"..slotName).red:Hide()
				else
					bai_slotItems[slotName]["indicator"] = 0
					getglobal("BAI_"..slotName):Hide()
					getglobal("BAI_"..slotName).yellow:Hide()
					getglobal("BAI_"..slotName).red:Hide()
				end

				-- SPECIAL CASE FOR dual slot artifact weapons
				-- The slot with the relics will report the correct item level
				-- The greatest value of main hand or secondary hand will be treated as iLvl for both
				if (slotName == "SecondaryHandSlot" and quality == 6) then
					-- check if secondary hand iLvl > main hand
					if (bai_slotItems[slotName]["iLvl"] > bai_slotItems["MainHandSlot"]["iLvl"]) then
						bai_slotItems["MainHandSlot"]["iLvl"] = bai_slotItems[slotName]["iLvl"]
						-- redo icon display for main hand
						if (bai_slotItems["MainHandSlot"]["iLvl"] < bai_currentAvgLvl) then
							bai_slotItems["MainHandSlot"]["indicator"] = 1
							getglobal("BAI_MainHandSlot"):Show()
							getglobal("BAI_MainHandSlot").yellow:Show()
							getglobal("BAI_MainHandSlot").red:Hide()
						else
							bai_slotItems["MainHandSlot"]["indicator"] = 0
							getglobal("BAI_MainHandSlot"):Hide()
							getglobal("BAI_MainHandSlot").yellow:Hide()
							getglobal("BAI_MainHandSlot").red:Hide()
						end
					else -- main hand has greater or equal iLvl
						bai_slotItems["SecondaryHandSlot"]["iLvl"] = bai_slotItems["MainHandSlot"]["iLvl"]
						-- redo icon display for secondary hand
						if (bai_slotItems["SecondaryHandSlot"]["iLvl"] < bai_currentAvgLvl) then
							bai_slotItems["SecondaryHandSlot"]["indicator"] = 1
							getglobal("BAI_SecondaryHandSlot"):Show()
							getglobal("BAI_SecondaryHandSlot").yellow:Show()
							getglobal("BAI_SecondaryHandSlot").red:Hide()
						else
							bai_slotItems[slotName]["indicator"] = 0
							getglobal("BAI_SecondaryHandSlot"):Hide()
							getglobal("BAI_SecondaryHandSlot").yellow:Hide()
							getglobal("BAI_SecondaryHandSlot").red:Hide()
						end
					end
				end

			end -- end if (iLevel ~= nil)
			
		end -- end if (itemId ~= nil)

	end -- end for loop
	
end


-----------------------------------------------------------------------------
-- Loops through each item in bai_slotItems array that will display an 
-- indicator and dertermines if "lowest" indicator should be shown on the 
-- item. It is possible for there to be multiple "lowest" items.
-----------------------------------------------------------------------------
local function bai_UpdateLowestItems()
	local i, slotName
	local lowest_iLvl = 9999
	local bai_tempNameArray = {}
	
	-- reset global count
	bai_currentLowItemCount = 0

	-- Loop through each item of array
	for i,slotName in ipairs(bai_slotNameArray) do 
		-- check if indicator is 1 - slot is below average
		if (bai_slotItems[slotName]["indicator"] == 1) then
			-- check for lowest iLvl
			if (bai_slotItems[slotName]["iLvl"] < lowest_iLvl) then
				lowest_iLvl = bai_slotItems[slotName]["iLvl"]
				bai_tempNameArray = {}
				table.insert(bai_tempNameArray, slotName)
			elseif (bai_slotItems[slotName]["iLvl"] == lowest_iLvl) then
				table.insert(bai_tempNameArray, slotName)
			end
			-- increment global count
			bai_currentLowItemCount = bai_currentLowItemCount + 1
		end
	end

	-- May be multiple lowest level items - loop through and update the graphic from yellow to red
	if (#bai_tempNameArray > 0) then
		for i,slotName in ipairs(bai_tempNameArray) do 
			getglobal("BAI_"..slotName).yellow:Hide()
			getglobal("BAI_"..slotName).red:Show()
		end
	end
	
end


-----------------------------------------------------------------------------
-- Called when OnEnter event is triggered over an indicator image.
-----------------------------------------------------------------------------
function BAI_Indicator_OnEnter(self)
	local name = self:GetName()
	
	-- remove "BAI_" from name to get slotname
	local slotName = string.sub(name, 5)
	local iLvl = bai_slotItems[slotName]["iLvl"]
	local displayStr = HIGHLIGHT_FONT_COLOR_CODE .. L.ITEM_LEVEL .. FONT_COLOR_CODE_CLOSE .. RED_FONT_COLOR_CODE .. iLvl .. FONT_COLOR_CODE_CLOSE
	
	bai_DebugOut("BAI_Indicator_OnEnter: ".. name)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:AddLine(displayStr)
	displayStr = HIGHLIGHT_FONT_COLOR_CODE .. L.BELOW_AVERAGE .. FONT_COLOR_CODE_CLOSE .. GREEN_FONT_COLOR_CODE .. bai_currentAvgLvl .. FONT_COLOR_CODE_CLOSE
	GameTooltip:AddLine(displayStr)
	GameTooltip:Show()

end


-----------------------------------------------------------------------------
-- Called when OnShow event is triggered from addon.
-----------------------------------------------------------------------------
function BAI_OnShow(self)
	local name = self:GetName()
	
	bai_DebugOut("BAI_OnShow: ".. name)

	if (bai_RefreshAvgLvl == 1) then
		bai_UpdateEquippedLvl()
		bai_UpdateEquippedItems()
		bai_UpdateLowestItems()
		bai_RefreshAvgLvl = 0
	end
end


-----------------------------------------------------------------------------
-- Event handler for addon.
-- 
-- handled events: 		ADDON_LOADED
--						PLAYER_EQUIPMENT_CHANGED
-----------------------------------------------------------------------------
function BAI_OnEvent(self, event, ...)

	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11 = ...
	
	bai_DebugOut("BAI_OnEvent: ".. arg1 .. " " .. event)

	-- Things to do after addon is loaded
	if (event == "ADDON_LOADED") then
		if (arg1 == "BelowAverageItems") then
			-- current iLvl not available at this point - so hold off to OnShow
		end
	end
	
	-- Equpment changed - recheck for below average gear
	if (event == "PLAYER_EQUIPMENT_CHANGED") then
		local slot, hasItem = ...
		if hasItem == nil then 
			hasItem = 0
		end
		bai_DebugOut(event .. " slot: " .. slot .. " hasItem: " .. tostring(hasItem))
		-- update current iLvl average
		bai_UpdateEquippedLvl()
		-- update current items
		bai_UpdateEquippedItems()
		-- update lowest items
		bai_UpdateLowestItems()
	end

end


-----------------------------------------------------------------------------
-- Called when addon receives OnLoad event 
-----------------------------------------------------------------------------
function BAI_OnLoad(self)

	bai_DebugOut("BAI_OnLoad: "..self:GetName())

	-- Register Events
	self:RegisterEvent("ADDON_LOADED")
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
	
	-- EVENT DEBUGGING - as needed
	--self:RegisterAllEvents()
	--self:UnregisterEvent("CHAT_MSG_CHANNEL")
	--self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	-- init variables
	bai_RefreshAvgLvl = 1
	
end
